#ifndef CMANAGECACHEDIALOG_H
#define CMANAGECACHEDIALOG_H
#include <gtkmm.h>
#include <iostream>
#include <vector>
#include <sstream>
#include "CVersionCompare.h"
#include "common_fun.h"
using namespace Common;

// This class provides an interface in which the user can clear the
// packages from their cache.
// See COPYING for license information.

class CManageCacheDialog : public Gtk::Dialog
{
public:
	CManageCacheDialog(Glib::ustring dir);
protected:
	void	collectData(Glib::ustring& dir);
	void	clearCache();
	void	cleanCache();

	Gtk::Label		_TextLbl;
	Gtk::ProgressBar	_Progress;

	std::vector<Glib::ustring>	_Files;
};

CManageCacheDialog::CManageCacheDialog(Glib::ustring dir) :
_TextLbl("Press Clear to delete all the packages in the cache,\nor press Clean to remove old versions of packages.")
{
	set_title("Manage Package Cache");
	set_border_width(6);
	get_vbox()->set_spacing(6);
	get_vbox()->pack_start(_TextLbl);
	get_vbox()->pack_start(_Progress);
	add_button(Gtk::Stock::CLOSE, -4);
	add_button("_Clear", 1);
	add_button("_Clean", 2);
	set_default_response(-4);
	get_action_area()->set_layout(Gtk::BUTTONBOX_EDGE);
	show_all_children();

	while ( 1 ) {	// Keep going until the user clicks Close.
		collectData(dir);
		switch ( run() ) {
			case 1:
				clearCache();
				system(std::string("rm -rf " + dir + "/*").c_str());
				break;
			case 2:
				cleanCache();
				break;
			case -4:
				return;
		}
		set_response_sensitive(1, false);
		set_response_sensitive(2, false);
		_TextLbl.set_text("Press Close to continue");
	}
}

void CManageCacheDialog::collectData(Glib::ustring& dir)
{
	// Create a list of all the files in the cache.
	Glib::ustring	_cmd = "find " + dir + " -regex \".*\\.t[glbx]z\" > " + TMPFILE("zendo-cacheclear.tmp");
	system(_cmd.c_str());
	_Files.clear();

	std::ifstream			_ifcache;
	std::string			_tmpin;
	_ifcache.open(TMPFILE("zendo-cacheclear.tmp").c_str(), std::ios_base::in);
	while ( !_ifcache.eof() && _ifcache.is_open() ) {
		getline(_ifcache, _tmpin);
		if ( _tmpin.length() > 0 ) _Files.push_back(_tmpin);
	}
	_ifcache.close();

	_Progress.set_text(IntToString(_Files.size()) + " package(s) in cache");
}

void CManageCacheDialog::clearCache()
{	// Clear the package cache.
	WRITELOG("Clearing package cache...");
	_TextLbl.set_text("Clearing package cache...");

	// Delete the files while updating the progress bar.
	for ( int i = 0; i < _Files.size(); i++ ) {
		_Progress.set_fraction((float)(i + 1) / _Files.size());
		_Progress.set_text(_Files[i]);
		while ( Gtk::Main::events_pending() ) Gtk::Main::iteration();	
		Glib::ustring	_cmd = "rm -f " + _Files[i];
		system(_cmd.c_str());
	}

	WRITELOG("Deleted " + IntToString(_Files.size()) + " package(s) from the cache");
}

void CManageCacheDialog::cleanCache()
{
	CVersionCompare	_vcmp;
	WRITELOG("Cleaning package cache...");
	_TextLbl.set_text("Cleaning package cache...");

	// Search for all the old versions of packages.
	std::vector<Glib::ustring>	_rems;
	for ( int i = 0; i < _Files.size(); i++ ) {
		Glib::ustring	_curpkg = GetPackageName(_Files[i].substr(_Files[i].rfind('/') + 1));
		Glib::ustring	_curver = GetPackageVersion(_Files[i].substr(_Files[i].rfind('/') + 1));

		for ( int k = 0; k < _Files.size(); k++ ) {	// Search for a package newer than this 'i' one.
			if ( !_Files[k].length() ) continue;
			Glib::ustring	_pkg = GetPackageName(_Files[k].substr(_Files[k].rfind('/') + 1));
			Glib::ustring	_ver = GetPackageVersion(_Files[k].substr(_Files[k].rfind('/') + 1));

			if ( k != i && _curpkg == _pkg && _vcmp.Compare(_ver, _curver) ) {
				_rems.push_back(_Files[i]);
				_Files[i] = "";
				break;
			}
		}
	}

	// Delete the files while updating the progress bar.
	for ( int i = 0; i < _rems.size(); i++ ) {
		_Progress.set_fraction((float)(i + 1) / _rems.size());
		_Progress.set_text(_rems[i]);
		while ( Gtk::Main::events_pending() ) Gtk::Main::iteration();	
		Glib::ustring	_cmd = "rm -f " + _rems[i];
		system(_cmd.c_str());
	}

	WRITELOG("Deleted " + IntToString(_rems.size()) + " package(s) from the cache");
}

#endif
